<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Extra Fee for Magento 2
 */

namespace Amasty\Extrafee\Controller\Adminhtml\Index;

use Magento\Backend\App\Action;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\DataObjectFactory;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Serialize\Serializer\FormData;

class Validate extends Action
{
    /**
     * @var DataObjectFactory
     */
    private $dataObjectFactory;

    /**
     * @var FormData
     */
    private $formDataSerializer;

    public function __construct(
        Action\Context    $context,
        DataObjectFactory $dataObjectFactory,
        FormData          $formDataSerializer
    ) {
        parent::__construct($context);
        $this->dataObjectFactory = $dataObjectFactory;
        $this->formDataSerializer = $formDataSerializer;
    }

    /**
     * @return ResponseInterface | ResultInterface
     */
    public function execute()
    {
        /** @var \Magento\Framework\DataObject $response */
        $response = $this->dataObjectFactory->create();
        $response->setError(false);

        $data = $this->getRequest()->getParams();
        $unserializedOptions = $this->formDataSerializer->unserialize($data['serialized_options']);
        $data = array_merge($data, $unserializedOptions);

        try {
            if (isset($data['option'])) {
                $options = $data['option'];
                foreach ($options['value'] as $labels) {
                    if (empty($labels[0])) {
                        throw new LocalizedException(__('The value of Admin scope can\'t be empty.'));
                    }
                }
            }
        } catch (\Exception $e) {
            $response->setError(true);
            $response->setMessage($e->getMessage());
        }

        return $this->resultFactory->create(ResultFactory::TYPE_JSON)->setJsonData($response->toJson());
    }
}
