<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Extra Fee for Magento 2
 */

namespace Amasty\Extrafee\Plugin\Paypal\Braintree\Request;

use Braintree\TransactionLineItem;
use Magento\Payment\Gateway\Helper\SubjectReader;
use Magento\Quote\Model\QuoteRepository;
use PayPal\Braintree\Gateway\Request\Level23ProcessingDataBuilder;

class Level23ProcessingDataBuilderPlugin
{
    /**
     * @var SubjectReader
     */
    private SubjectReader $subjectReader;

    /**
     * @var quoteRepository
     */
    private QuoteRepository $quoteRepository;

    public function __construct(
        SubjectReader $subjectReader,
        QuoteRepository $quoteRepository
    ) {
        $this->subjectReader = $subjectReader;
        $this->quoteRepository = $quoteRepository;
    }

    /**
     * @param Level23ProcessingDataBuilder $subject
     * @param array $result
     * @param array $buildSubject
     * @return array
     */
    public function afterBuild(
        Level23ProcessingDataBuilder $subject,
        $result,
        $buildSubject
    ) {
        $paymentDO = $this->subjectReader->readPayment($buildSubject);
        $order = $paymentDO->getOrder();
        $quote = $this->quoteRepository->get($order->getQuoteId());

        if (array_key_exists('amasty_extrafee', $quote->getTotals()) &&
            $quote->getTotals()['amasty_extrafee']->getValue()
        ) {
            $extraFee = $this->numberToString($quote->getTotals()['amasty_extrafee']->getValue(), 2);
            $extraFeeLineItem = [
                'name' => 'Amasty Extra Fee for Order',
                'kind' => TransactionLineItem::DEBIT,
                'quantity' => 1.00,
                'unitAmount' => $extraFee,
                'totalAmount' => $extraFee
            ];

            $result['lineItems'][] = $extraFeeLineItem;
        }

        return $result;
    }

    /**
     * @param float $num
     * @param int $precision
     * @return string
     */
    private function numberToString($num, int $precision): string
    {
        // To counter the fact that Magento often wrongly returns a sting for price values, we can cast it to a float.
        if (is_string($num)) {
            $num = (float) $num;
        }

        return (string) round($num, $precision);
    }
}
