/**
 * Init Options Panel
 */
define([
    'jquery',
    'uiRegistry',
    'mage/translate'
], function ($, registry, $t) {
    'use strict';

    var panelId = 'manage-options-panel',
        initOptions = {
            selectors: {
                editForm: '#edit_form',
                optionCount: '#option-count-check',
                validationInput: '[data-amxexfee-js="required-options"]',
                priceTypesSelect: '[data-item-select-id="%s"]',
                optionsContainer: '[data-amexfee-js="options-container"]',
                options: '[data-amexfee-js="options"]',
                priceTypes: '[data-amexfee-js="price-types"]',
                saveButtons: '[data-ui-id="amasty-extrafee-edit-content-save-button"],' +
                    '[data-ui-id="amasty-extrafee-edit-content-saveandcontinue-button"]',
                option: 'tr:not(.no-display) .adminview',
                messageContainer: '.messages:first'
            },
            errorContainer: document.createElement('label'),

            /**
             * Init options for fee item
             *
             * @return {void}
             */
            initialize: function () {
                this.initSelectors();
                this.renderOptions();
                this.initEventHandlers();
                this.renderErrorContainer();
            },

            /**
             * @return {void}
             */
            initSelectors: function () {
                this.optionsContainer = $(this.selectors.optionsContainer);
                this.editForm = $(this.selectors.editForm);
                this.validationInput = $(this.selectors.validationInput);
                this.optionCount = $(this.selectors.optionCount);
                this.saveButtons = $(this.selectors.saveButtons);
                this.messageContainer = $(this.selectors.messageContainer);
            },

            /**
             * @return {void}
             */
            renderOptions: function () {
                var self = this;

                this.optionsContainer.trigger('render');
                $(self.selectors.priceTypes).each(function () {
                    var input = $(this),
                        id = input.data('itemId'),
                        selectorSelectElement = self.selectors.priceTypesSelect.replace('%s', id);

                    $(selectorSelectElement).val(input.val());
                });
                this.optionsContainer.sortable({
                    cancel: 'select, input, button'
                });
            },

            /**
             * @return {void}
             */
            initEventHandlers: function () {
                this.editForm.on('beforeSubmit', this.validate.bind(this));
                this.saveButtons.on('click', this.validateOptions.bind(this));

                this.eventOrderChange();
            },

            /**
             * @return {void}
             */
            eventOrderChange: function (){
                $.each(this.saveButtons, function (index, button) {
                    this._swapSubmit($(button));
                }.bind(this));
            },

            /**
             * @param {Element} button
             * @private
             * @returns {void}
             */
            _swapSubmit: function (button) {
                var $button = $(button),
                    listeners;

                listeners = $._data($button[0], 'events').click;
                if (listeners) {
                    listeners.unshift(listeners.pop());
                }
            },

            /**
             * @return {void}
             */
            renderErrorContainer: function () {
                this.errorContainer.innerHTML = $t('This is a required field.');
                this.errorContainer.classList.add('mage-error');
                this.errorContainer.setAttribute('id', '-error');
            },

            /**
             * @return {void}
             */
            renderMessageContainer: function () {
                $('.page-columns').before($(this.messageContainer)[0])
            },

            /**
             * @return {Boolean}
             */
            validateOptions: function () {
                var fieldsCounter = 0,
                    fieldRealCount = $(this.selectors.option).length;

                if (this.optionCount === '' || fieldRealCount === 0) {
                    return true;
                }

                this.renderMessageContainer();

                for (var i = 0; i < fieldRealCount; i++) {
                    if ($(this.selectors.option)[i].value === '') {
                        this.errorContainer.setAttribute('for', $(this.selectors.option)[i].name);
                        $(this.selectors.option)[i].after(this.errorContainer);
                        continue;
                    }
                    $('[for="' + $(this.selectors.option)[i].name + '"]').hide();
                    fieldsCounter++;
                }

                return fieldsCounter === fieldRealCount
            },

            /**
             * @return {void}
             */
            validate: function () {
                this.validationInput.val(this.isValid());
            },

            /**
             * @return {String}
             */
            isValid: function () {
                return this.optionCount.val();
            }
        };

    registry.get(panelId, initOptions.initialize.bind(initOptions));

    return initOptions;
});
